/**
 * Exact-Sort algorithm demonstration. 
 * Exact-Sort is a sort algorithm which changes the positions of elements only once. 
 * The idea is : Locating the elements directly to the their ordered positions. 
 * To find the exact positions of the elements it counts the smaller elements than the current element. 
 * After locating an element it locates the old holder of the position.
 * 
 * @author Ahmet Akkk
 * @version 	@(#)ExactSortAlgorithm.java	1.2, November 2002
 */
public class ExactSortAlgorithm extends SortAlgorithm 
{

        /**
         * it sorts the integer arrays by using the Exact-Sort algorithm.
         *
         * @param the integer array.
         */    
	public void sort(int a[]) throws Exception
	{          
                
                // this boolean array indicates which elemnt is sorted which is not. False means not sorted yet.
                //In the begining all is false.
                boolean[] b = new boolean[a.length];
                
                // the destination (sorted position) of the current element.                
		int d = -1;              
                
                // starting point where to start to sort? 
                // we can give it manually as well.
		int sp = firstFalse(b);  
                
                // the current element.
		int t = sp;
                
                //the value of the current element.
		int tv =a[t];		
		
                //our loop we know the max loop number is element count of array.
                //we can use for(int i=0; i<a.length; i++) as well.
		while(true) 
		{                                           
                        // get the current element destination (sorted position).
			d = getDestination(a, b, tv ,sp, t);
			
                        // there is no unsorted element anymore
			if (d == -1)
				return;				
			
			//anyway end of this loop it will be sorted.
			b[d] = true;				
			
                        //tination is equals to starting point. We have to find new starting point after locating the current element.
			if (d == sp)
			{
                                // if there is no coincidence in other words if it is not in the destination in the first place
				if (d != t)
                                        //locate the element.
					a[d] = tv;		
                                                                
                                // take the fisrt unsorted elemnt as a starting point.
				sp = firstFalse(b); 
                                
                                // if there is no unsorted elements any more
                                if(sp == -1)
                                    break;                      
                                
                                // setting the current element.
                                tv = a[sp];
                                t = sp;                                
        		}
			else
			{
                                // stating the old position holder.
				int temp = a[d];
                                
                                //locate the element.
				a[d] = tv;
                                
                                // setting the current element.
				tv = temp;
				t = d;				
			}
			
		}
                //to paint the applet.
		pause(t, 0);			
	}

        /**
         * it gives the destination (ordered position) of the given element.
         * @param  a    the integer array.
         * @param  b    the boolean array which indicates the sorted and un sorted elements
         * @param  v    the value of current element.
         * @param  sp   starting point.
         * @param t     is just used for pause method of SortAlgorithm. It is not related with Exact-Sort algrithm.
         */ 
	protected int getDestination(int[] a, boolean[] b, int v, int sp, int t) throws Exception
	{                 
                // counter. It counts the unsorted and smaller elements than the given element.
		int c = 0;
		
		for (int i = a.length; --i >= 0; )
		{                    
			if ( !b[i] && a[i]< v && i != sp )
				c++;
                        //to paint the applet.
                        pause(t , i);
		}	
		
                // return the destination.
                return realIndex(b, c);
	}


        /**
         * it returns the index of the given numbered unsorted element.
         * For example if you give 0 as a param it returns the first unsorted element.
         *
         * @param b the boolean array which indicates the sorted and un sorted elements
         * @param f the number of the false element
         */
	protected int realIndex(boolean[] b, int f)
	{                                
		int fc = -1;
		
		for (int i = 0;  i<b.length; i++ )
		{
			if (!b[i])	
				fc++;
				
			if(fc == f)
				return i;				
		}
		return -1;
	}
   
        
        /**
         * it returns the index of the first unsorted element.
         * @param b the boolean array which indicates the sorted and un sorted elements         
         */        
	protected int firstFalse( boolean[] b )
	{            
                return realIndex(b, 0);
	}      
}
